#Collect data and Define y and x's variables
data4project<-read.csv("C:/Users/lhwiles/Downloads/housedata.csv",head=T)
attach(data4project)

### Split data into training and test ,60% used for training, 40% used for testing 
traindata<-data4project[1:12967,]
testdata<-data4project[12968:nrow(data4project),]

### Better way of dividing data to ensure randomness
indexes = sample(1:nrow(data4project), size=0.6*nrow(data4project))
traindata<-data4project[indexes,]
testdata<-data4project[-indexes,]


###Define variables
y0<-traindata$price
x1<-traindata$id
x2<-traindata$date
x3<-traindata$bedrooms
x4<-traindata$bathrooms
x5<-traindata$sqft_living
x6<-traindata$sqft_lot
x7<-traindata$floors
x8<-traindata$waterfront
x9<-traindata$view
x10<-traindata$condition
x11<-traindata$grade
x12<-traindata$sqft_above
x13<-traindata$sqft_basement
x14<-traindata$yr_built
x15<-traindata$yr_renovated
x16<-traindata$zipcode
x17<-traindata$lat
x18<-traindata$long
x19<-traindata$sqft_living15
x20<-traindata$sqft_lot15

#preliminary checks on data quality
m1<-lm(y0~x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x12+x13+x14+x15+x16+x17+x18+x19+x20)
res1<-residuals(m1)
par(mfrow=c(2,2))
plot(m1)
anova(m1)
summary(m1)

##Check Multicollinearity using Variance Inflation Factor 
library(car)
vif(m1)
alias(m1)
plot(x13~x12, xlab="Sqft above",ylab="Sqft of basement")
plot(x13~x5, xlab="Sqft of Living",ylab="Sqft of basement")
plot(x5~x12, xlab="Sqft above",ylab="Sqft of Living")

#Check for normality by using Shapiro-Wilk normality test : Failed due to sample size
shapiro.test(res1)

##scatter plot& correlation matrix
data1<-as.matrix(traindata)
pairs(data1)
View(cor(data1))


m12<-lm(y0~x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x12+x14+x15+x16+x17+x18+x19+x20)
vif(m12)

m13<-lm(y0~x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20)
vif(m13)
par(mfrow=c(2,2))
plot(m13)

#Check error variance constancy
library(lmtest)
bptest(m13)

###Box-Cox transformation 
library(MASS)
boxcox(m12,xlab=expression(lambda[y0]),plotit=T)
boxcox(m12,xlab=expression(lambda[y0]),plotit=F)

m2<-lm(log(y0)~x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20)
par(mfrow=c(2,2))
plot(m2)
summary(m2)
anova(m2)

#check for outliers
outlierTest(m2)


###Model subset Selection
library(leaps)
m3<-regsubsets(log(y0)~x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20,data=traindata,nbest=1)
summary(m3)
par(mfrow=c(1,1))
plot(m3,scale="adjr2")
plot(m3,scale="Cp")
plot(m3,scale="bic", main = "BIC Subset Selection" )

###stepwise selection based on AIC##
m4<- lm(log(y0) ~ 1)

ansf1 <- step(m4, scope=list(lower= ~ 1,
         upper=~ x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20), direction="forward", data=traindata,trace=1)

#stepwise selection based on BIC adding k=log(n)
ansf2 <- step(m4, scope=list(lower= ~ 1,
         upper=~ x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20), direction="forward", data=traindata,k=log(length(y0)),trace=1)

#Backward elimination based on AIC#
ansf3 <- step(m2, scope=list(lower= ~ 1,
         upper=~ x1+x2+x3+x4+x5+x6+x7+x8+x9+x10+x11+x14+x15+x16+x17+x18+x19+x20),
                direction="backward",  data=traindata)

####model validation##########################################

#testing dataset 
t_y0<-testdata$price
t_x1<-testdata$id
t_x2<-testdata$date
t_x3<-testdata$bedrooms
t_x4<-testdata$bathrooms
t_x5<-testdata$sqft_living
t_x6<-testdata$sqft_lot
t_x7<-testdata$floors
t_x8<-testdata$waterfront
t_x9<-testdata$view
t_x10<-testdata$condition
t_x11<-testdata$grade
t_x12<-testdata$sqft_above
t_x13<-testdata$sqft_basement
t_x14<-testdata$yr_built
t_x15<-testdata$yr_renovated
t_x16<-testdata$zipcode
t_x17<-testdata$lat
t_x18<-testdata$long
t_x19<-testdata$sqft_living15
t_x20<-testdata$sqft_lot15

bestm1<-lm(log(t_y0)~t_x4+t_x5+t_x9+t_x10+t_x11+t_x14+t_x17+t_x19)	
bestm2<-lm(log(t_y0)~t_x4+t_x5+t_x9+t_x11+t_x14+t_x17+t_x19)
bestm3<-lm(log(t_y0)~t_x5+t_x9+t_x11+t_x14+t_x17+t_x19)

summary(bestm1)
summary(bestm2)
summary(bestm3)

anova(bestm1)
anova(bestm2)
anova(bestm3)
extractAIC(bestm1)
extractAIC(bestm2)
extractAIC(bestm3)
extractAIC(bestm1, k=log(length(y0)))
extractAIC(bestm2, k=log(length(y0)))
extractAIC(bestm3, k=log(length(y0)))

#model 1 prediction
y1<-predict(bestm1,newdata=data.frame(x4=t_x4,x5=t_x5,x8=t_x8,x9=t_x9,x10=t_x10,x11=t_x11,x14=t_x14,x17=t_x17,x19=t_x19))
#model 2 prediction
y2<-predict(bestm2,newdata=data.frame(x4=t_x4,x5=t_x5,x9=t_x9,x11=t_x11,x14=t_x14,x17=t_x17,x19=t_x19))
#model 3 prediction
y3<-predict(bestm3,newdata=data.frame(x5=t_x5,x9=t_x9,x11=t_x11,x14=t_x14,x17=t_x17,x19=t_x19))

n<-length(t_y0)
MSPR1<-sum((log(t_y0)-y1)^2)/n
MSPR2<-sum((log(t_y0)-y2)^2)/n
MSPR3<-sum((log(t_y0)-y3)^2)/n

MSPR1
MSPR2
MSPR3


####Final Model######

Fy0<-data4project$price
Fx1<-data4project$id
Fx2<-data4project$date
Fx3<-data4project$bedrooms
Fx4<-data4project$bathrooms
Fx5<-data4project$sqft_living
Fx6<-data4project$sqft_lot
Fx7<-data4project$floors
Fx8<-data4project$waterfront
Fx9<-data4project$view
Fx10<-data4project$condition
Fx11<-data4project$grade
Fx12<-data4project$sqft_above
Fx13<-data4project$sqft_basement
Fx14<-data4project$yr_built
Fx15<-data4project$yr_renovated
Fx16<-data4project$zipcode
Fx17<-data4project$lat
Fx18<-data4project$long
Fx19<-data4project$sqft_living15
Fx20<-data4project$sqft_lot15

m_final<-lm(log(Fy0)~Fx4+Fx5+Fx9+Fx10+Fx11+Fx14+Fx17+Fx19)
summary(m_final)
par(mfrow=c(2,2))
plot(m_final)

res_final<-residuals(m_final)
par(mfrow=c(1,1))
qqnorm(res_final)
qqline(res_final)
vif(m_final)
bptest(m_final)















